/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.awt.Window;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockableFrame;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IPerspectiveFactory;
import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IViewReference;
import com.onaro.client.leekui.ui.PartInitException;
import com.onaro.client.leekui.ui.PlatformUI;
import com.onaro.client.leekui.ui.WorkbenchException;
import com.onaro.client.leekui.ui.internal.compatibility.PerspectiveDescriptor;
import com.onaro.client.leekui.ui.internal.misc.StatusUtil;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.Perspective
 */
public class Perspective {

	final static private String VERSION_STRING = "0.0002"; //$NON-NLS-1$

	private PerspectiveHelper presentation;

	IMemento memento;

	final WorkbenchPage page;

	private PerspectiveDescriptor descriptor;

	IPerspectiveFactory factory;

	public Perspective(PerspectiveDescriptor descriptor, WorkbenchPage page) throws WorkbenchException {
		this.descriptor = descriptor;
		this.page = page;

		if (descriptor != null) {
			createPresentation(descriptor);
		}
	}

	public final PerspectiveHelper getPresentation() {
		if (presentation == null) {
			presentation = new PerspectiveHelper(this);
		}

		return presentation;
	}

	public PerspectiveDescriptor getDescriptor() {
		return descriptor;
	}

	public IViewReference findView(String viewId) {
		return findView(viewId, null);
	}

	private ViewFactory getViewFactory() {
		return page.getViewFactory();
	}

	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public List<IViewReference> getViewReferences() {
		List<IViewReference> viewReferences = new ArrayList<IViewReference>();

		DefaultDockingManager dockingManager = getDockingManager();

		List<String> frameNames = dockingManager.getAllFrameNames();

		for (String frameName : frameNames) {
			DockableFrame dockableFrame = dockingManager.getDockableFrame(frameName);

			IViewReference viewReference = WorkbenchPageUtils.getViewReference(dockableFrame);

			if (viewReference != null) {
				viewReferences.add(viewReference);
			}
		}

		return viewReferences;
	}

	public IViewReference findView(String id, String secondaryId) {
		for (IViewReference ref : getViewReferences()) {
			if (id.equals(ref.getId())
					&& (secondaryId == null ? ref.getSecondaryId() == null : secondaryId.equals(ref.getSecondaryId()))) {
				return ref;
			}
		}
		return null;
	}

	public IViewPart showView(String viewId, String secondaryId) throws PartInitException {

		ViewFactory factory = getViewFactory();
		ViewReference ref = factory.createView(viewId, secondaryId);
		IViewPart part = (IViewPart) ref.getPart(true);
		if (part == null) {
			String msg = null;
			throw new PartInitException(msg);
		}

		DefaultDockingManager dockingManager = getDockingManager();

		ViewPane viewPane = (ViewPane) ref.getPane();

		DockableFrame dockableFrame = getPresentation().getDockableFrame(viewPane, true);
		// removed because we don't want floating
		// dockableFrame.setInitMode(DockContext.STATE_FLOATING);

		//add only if the docking manager does not already contain this frame
		if (dockingManager.getFrame(dockableFrame.getKey()) == null)
		{
			dockingManager.addFrame(dockableFrame);
		}

		return part;
	}

	private void createPresentation(PerspectiveDescriptor perspectiveDescriptor) throws WorkbenchException {
		if (perspectiveDescriptor.hasCustomDefinition()) {
			// loadCustomPersperspective(persp);
		}
		else {
			loadPredefinedPerspective(perspectiveDescriptor);
		}
	}

	private void loadPredefinedPerspective(PerspectiveDescriptor perspectiveDescriptor) throws WorkbenchException {
		try {
			factory = perspectiveDescriptor.createFactory();
		}
		catch (Exception e) {
			throw new WorkbenchException(e);
		}

		/*
		 * IPerspectiveFactory#createFactory() can return null
		 */
		if (factory == null) {
			String msg = null;
			throw new WorkbenchException(msg);
		}
	}

	public DefaultDockingManager getDockingManager() {
		return page.getDockingManager();
	}

	protected void onActivate() {

		getPresentation().activate();
	}

	protected void onDeactivate() {

		PerspectiveHelper presentation = getPresentation();
		presentation.deactivate();

	}

	public IStatus restoreState() {

		if (this.memento == null) {
			return new Status(IStatus.OK, PlatformUI.PLUGIN_ID, 0, "", null); //$NON-NLS-1$
		}

		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK,
				WorkbenchMessages.Perspective_problemsRestoringPerspective, null);

		IMemento memento = this.memento;
		this.memento = null;

		final DefaultDockingManager dockingManager = getDockingManager();

		// to fix a jide bug where the docking manager's content container is not the correct size
		Runnable runnable = new Runnable() {
			public void run() {
				Window window = (Window) dockingManager.getRootPaneContainer();

				window.validate();
			}
		};

		SwingUtilities.invokeLater(runnable);

		IMemento layoutMemento = memento.getChild(IWorkbenchConstants.TAG_LAYOUT);
		result.merge(this.getPresentation().restoreState(layoutMemento));

		dockingManager.setShowWorkspace(memento.getInteger(IWorkbenchConstants.TAG_AREA_VISIBLE) > 0);

		return result;
	}

	public IStatus restoreState(IMemento memento) {

		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK,
				WorkbenchMessages.Perspective_problemsRestoringPerspective, null);

		// Create persp descriptor.
		descriptor = new PerspectiveDescriptor(null, null, null, false);
		result.add(descriptor.restoreState(memento));
		PerspectiveDescriptor desc = Workbench.getInstance().getPerspectiveRegistry().findPerspectiveWithId(
				descriptor.getId());
		if (desc != null) {
			descriptor = desc;
		}

		this.memento = memento;
		// Add the visible views.
		IMemento views[] = memento.getChildren(IWorkbenchConstants.TAG_VIEW);
		result.merge(createReferences(views));

		memento = memento.getChild(IWorkbenchConstants.TAG_FAST_VIEWS);
		if (memento != null) {
			views = memento.getChildren(IWorkbenchConstants.TAG_VIEW);
			result.merge(createReferences(views));
		}
		return result;

	}

	private IStatus createReferences(IMemento views[]) {
		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK,
				WorkbenchMessages.Perspective_problemsRestoringViews, null);

		for (int x = 0; x < views.length; x++) {
			// Get the view details.
			IMemento childMem = views[x];
			String id = childMem.getString(IWorkbenchConstants.TAG_ID);
			// skip creation of the intro reference - it's handled elsewhere.
			// if (id.equals(IIntroConstants.INTRO_VIEW_ID)) {
			// continue;
			// }

			String secondaryId = ViewFactory.extractSecondaryId(id);
			if (secondaryId != null) {
				id = ViewFactory.extractPrimaryId(id);
			}
			// Create and open the view.
			try {
				if (!"true".equals(childMem.getString(IWorkbenchConstants.TAG_REMOVED))) { //$NON-NLS-1$
					page.getViewFactory().createView(id, secondaryId);
					ViewPane viewPane = WorkbenchPageUtils.createViewPane(page, id, secondaryId);
					DockableFrame dockableFrame = getPresentation().getDockableFrame(viewPane, true);
					getDockingManager().addFrame(dockableFrame);
				}
			}
			catch (PartInitException e) {
				childMem.putString(IWorkbenchConstants.TAG_REMOVED, "true"); //$NON-NLS-1$
				result.add(StatusUtil.newStatus(IStatus.ERROR, e.getMessage() == null ? "" : e.getMessage(), e)); //$NON-NLS-1$
			}
		}
		return result;
	}

	public IStatus saveState(IMemento memento) {
		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK,
				WorkbenchMessages.Perspective_problemsSavingPerspective, null);

		result.merge(saveState(memento, descriptor, true));

		return result;
	}

	/**
	 * Save the layout.
	 */
	private IStatus saveState(IMemento memento, PerspectiveDescriptor p, boolean saveInnerViewState) {
		MultiStatus result = new MultiStatus(PlatformUI.PLUGIN_ID, IStatus.OK,
				WorkbenchMessages.Perspective_problemsSavingPerspective, null);

		if (this.memento != null) {
			memento.putMemento(this.memento);
			return result;
		}

		// Save the version number.
		memento.putString(IWorkbenchConstants.TAG_VERSION, VERSION_STRING);
		result.add(p.saveState(memento));

		// Save the views.
		int errors = 0;
		for (ViewPane pane : presentation.getViewPanes()) {
			IViewReference ref = pane.getViewReference();
			IMemento viewMemento = memento.createChild(IWorkbenchConstants.TAG_VIEW);
			viewMemento.putString(IWorkbenchConstants.TAG_ID, ViewFactory.getKey(ref));
		}

		DefaultDockingManager dockingManager = this.getDockingManager();

		// Save the editor visibility state
		if (dockingManager.isShowWorkspace()) {
			memento.putInteger(IWorkbenchConstants.TAG_AREA_VISIBLE, 1);
		}
		else {
			memento.putInteger(IWorkbenchConstants.TAG_AREA_VISIBLE, 0);
		}

		// Save the fixed state
		if (PageLayout.isFixed(page)) {
			memento.putInteger(IWorkbenchConstants.TAG_FIXED, 1);
		}
		else {
			memento.putInteger(IWorkbenchConstants.TAG_FIXED, 0);
		}

		IMemento layoutMemento = memento.createChild(IWorkbenchConstants.TAG_LAYOUT);
		result.merge(this.getPresentation().saveState(layoutMemento));

		if (errors > 0) {
			String message = WorkbenchMessages.Perspective_multipleErrors;
			if (errors == 1) {
				message = WorkbenchMessages.Perspective_oneError;
			}

			// MessageDialog.openError(null,WorkbenchMessages.Error, message);
		}

		return result;
	}

	/**
	 * Dispose the perspective and all views contained within.
	 */
	public void dispose() {
		// Get rid of presentation.
		if (presentation == null) {
			return;
		}

		presentation.deactivate();
		presentation.dispose();

		// fastViewPane.dispose();

		// Release each view.
		for (IViewReference viewRef : getViewReferences()) {
			getViewFactory().releaseView(viewRef);
		}

		// mapIDtoViewLayoutRec.clear();
	}
}
