/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import static com.onaro.client.leekui.ui.internal.WorkbenchPageUtils.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Icon;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockableFrame;
import com.jidesoft.docking.DockingManager;
import com.jidesoft.docking.event.DockableFrameAdapter;
import com.jidesoft.docking.event.DockableFrameEvent;
import com.jidesoft.utils.Base64;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.PlatformUI;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.PerspectiveHelper
 */
public class PerspectiveHelper {

	static final Logger logger = Logger.getLogger(PerspectiveHelper.class);

	private static final short DOCKING_MANAGER_VERSION = 1;

	private static final long serialVersionUID = 1L;

	private byte[] dockingManagerLayoutBytes;

	final Perspective perspective;

	private boolean active;

	public PerspectiveHelper(Perspective perspective) {

		if (perspective == null) throw new IllegalArgumentException();

		this.perspective = perspective;

		active = false;
	}

	public DefaultDockingManager getDockingManager() {
		return perspective.getDockingManager();
	}

	public void activate() {

		logDebug("Activating"); //$NON-NLS-1$

		try {
			if (dockingManagerLayoutBytes != null) {
				loadLayoutFromBytes();
			}
			else {
				loadLayoutFromFactory();
			}

			active = true;
		}
		catch (Exception e) {
			WorkbenchPlugin.log(e);
		}
	}

	public WorkbenchPage getPage() {
		return perspective.page;
	}

	private void loadLayoutFromFactory() {

		logDebug("Loading layout from factory"); //$NON-NLS-1$

		// clear out any data
		getDockingManager().resetToDefault();

		PageLayout layout = new PageLayout(perspective);

		WorkbenchPage.WorkbenchPageComponent component = getPage().getComponent();

		// turn off visibility to prevent flickering

		boolean wasVisible = component.isVisible();

		try {
			// TODO: JDM - this is breaking the parent's CardLayout, need to come up with an alternate way of 
			// avoiding flickering
//			component.setVisible(false);

			perspective.factory.createInitialLayout(layout);
		}
		finally {
			component.setVisible(wasVisible);
		}

		layout.setFixed(perspective.getDescriptor().getFixed());

		Runnable runnable = new Runnable() {
			public void run() {
				getPage().getComponent().validate();
			}
		};

		SwingUtilities.invokeLater(runnable);
	}

	public void deactivate() {

		if (!active) {
			return;
		}

		logDebug("Deactivating"); //$NON-NLS-1$

		try {
			saveLayoutToBytes();

		}
		catch (Exception e) {
			WorkbenchPlugin.log(e);
		}

		DefaultDockingManager dockingManager = getDockingManager();

		dockingManager.removeAllFrames();

		active = false;
	}

	public List<ViewPane> getViewPanes() {
		List<ViewPane> viewPanes = new ArrayList<ViewPane>();

		DefaultDockingManager dockingManager = getDockingManager();

		for (Object frameNameObj : dockingManager.getAllFrameNames()) {
			String frameName = (String) frameNameObj;

			DockableFrame frame = dockingManager.getFrame(frameName);

			ViewPane viewPane = getViewPane(frame);

			if (viewPane == null) continue;

			viewPanes.add(viewPane);
		}

		return viewPanes;
	}

	/**
	 * Get the dockable frame, make visible if it is not.
	 */
	public DockableFrame getDockableFrame(final ViewPane viewPane, boolean autoCreate) {

		if (viewPane == null) throw new IllegalArgumentException();

		ViewReference viewReference = viewPane.getViewReference();

		DefaultDockingManager dockingManager = getDockingManager();

		String dockableFrameKey = getDockableFrameKey(viewReference);

		DockableFrame dockableFrame = dockingManager.getFrame(dockableFrameKey);

		if (dockableFrame == null) {

			if (!autoCreate) return null;

			dockableFrame = new DockableFrame(dockableFrameKey);

			Icon viewIcon = viewReference.getTitleIcon();

			dockableFrame.setContentPane(viewPane);
			dockableFrame.setTabTitle(viewReference.getTitle());
			dockableFrame.setFrameIcon(viewIcon);

			// TODO: Rich - MUST RE-ADD THIS IN 4.0
			// dockableFrame.addDockableFrameListener(new DockableFrameListenerImpl());
		}

		// make sure the dockableframe has the correct view pane
		if (!viewPane.equals(getViewPane(dockableFrame))) {
			setViewPane(dockableFrame, viewPane);
		}

		return dockableFrame;
	}

	public void dispose() {
		// do nothing
	}

	public IStatus restoreState(IMemento memento) {

		try {
			String encodedBytes = memento.getTextData();
			dockingManagerLayoutBytes = Base64.decode(encodedBytes);

			loadLayoutFromBytes();
		}
		catch (Throwable t) {
			return new Status(Status.ERROR, PlatformUI.PLUGIN_ID, 0, "", t); //$NON-NLS-1$
		}

		return Status.OK_STATUS;

	}

	public IStatus saveState(IMemento memento) {
		try {
			saveLayoutToBytes();

			String encodedBytes = Base64.encodeBytes(dockingManagerLayoutBytes);
			memento.putTextData(encodedBytes);
		}
		catch (Throwable t) {
			return new Status(Status.ERROR, PlatformUI.PLUGIN_ID, 0, "", t); //$NON-NLS-1$
		}

		return Status.OK_STATUS;
	}

	private void loadLayoutFromBytes() throws Exception {

		logDebug("Load layout from bytes"); //$NON-NLS-1$

		if (dockingManagerLayoutBytes == null) return;

		final DockingManager dockingManager = getDockingManager();

		ByteArrayInputStream iStream = new ByteArrayInputStream(dockingManagerLayoutBytes);

		try {
			dockingManager.setVersion(DOCKING_MANAGER_VERSION);

			dockingManager.loadLayoutFrom(iStream);
		}
		finally {
			iStream.close();
		}

	}

	private void saveLayoutToBytes() throws Exception {

		logDebug("Saving layout to bytes"); //$NON-NLS-1$

		DockingManager dockingManager = getDockingManager();

		dockingManager.setVersion(DOCKING_MANAGER_VERSION);

		ByteArrayOutputStream oStream = new ByteArrayOutputStream(4 * 1024);

		dockingManager.saveLayoutTo(oStream);

		dockingManagerLayoutBytes = oStream.toByteArray();
	}

	void logDebug(String msg) {
		// TODO: use eclipse logging framework

		if (!logger.isDebugEnabled()) return;

		String prefix = perspective.toString();

		logger.debug(prefix + " - " + msg); //$NON-NLS-1$
	}

	class DockableFrameListenerImpl extends DockableFrameAdapter {
		public DockableFrameListenerImpl() {
			// do nothing
		}

		@Override
		public void dockableFrameHidden(DockableFrameEvent event) {

			DockableFrame dockableFrame = event.getDockableFrame();

			if (dockableFrame == null) return;

			ViewPane viewPane = getViewPane(dockableFrame);

			if (viewPane == null) return;

			// remove the view to the view pane
			setViewPane(dockableFrame, null);

			ViewReference viewReference = viewPane.getViewReference();

			if (viewReference == null) return;

			ViewFactory viewFactory = perspective.page.getViewFactory();
			viewFactory.releaseView(viewReference);
		}

	}
}
