/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * A ReferenceCounter is used to reference counting objects. Each object is identified by a unique ID. Together they
 * form an ID - value pair. An object is added to the counter by calling #put(id, object). From this point on additional
 * refs can be made by calling #addRef(id) or #removeRef(id).
 * 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.ReferenceCounter
 */
public class ReferenceCounter<K,V> {

	private Map<K,RefRec> mapIdToRec = new HashMap<K,RefRec>(11);

	/**
	 * Capture the information about an object.
	 */
	public class RefRec {
		public RefRec(K id, V value) {
			this.id = id;
			this.value = value;
			addRef();
		}

		public K getId() {
			return id;
		}

		public V getValue() {
			return value;
		}

		public int addRef() {
			++refCount;
			return refCount;
		}

		public int removeRef() {
			--refCount;
			return refCount;
		}

		public int getRef() {
			return refCount;
		}

		public boolean isNotReferenced() {
			return (refCount <= 0);
		}

		public K id;

		public V value;

		private int refCount;
	}

	/**
	 * Creates a new counter.
	 */
	public ReferenceCounter() {
		super();
	}

	/**
	 * Adds one reference to an object in the counter.
	 * 
	 * @param id
	 *            is a unique ID for the object.
	 * @return the new ref count
	 */
	public int addRef(K id) {
		RefRec rec = mapIdToRec.get(id);
		if (rec == null) {
			return 0;
		}
		return rec.addRef();
	}

	/**
	 * Returns the object defined by an ID. If the ID is not found <code>null</code> is returned.
	 * 
	 * @return the object or <code>null</code>
	 */
	public V get(K id) {
		RefRec rec = mapIdToRec.get(id);
		if (rec == null) {
			return null;
		}
		return rec.getValue();
	}

	/**
	 * Returns a complete list of the keys in the counter.
	 * 
	 * @return a Set containing the ID for each.
	 */
	public Set<K> keySet() {
		return mapIdToRec.keySet();
	}

	/**
	 * Adds an object to the counter for counting and gives it an initial ref count of 1.
	 * 
	 * @param id
	 *            is a unique ID for the object.
	 * @param value
	 *            is the object itself.
	 */
	public void put(K id, V value) {
		RefRec rec = new RefRec(id, value);
		mapIdToRec.put(id, rec);
	}

	/**
	 * @param id
	 *            is a unique ID for the object.
	 * @return the current ref count
	 */
	public int getRef(K id) {
		RefRec rec = mapIdToRec.get(id);
		if (rec == null) {
			return 0;
		}
		return rec.getRef();
	}

	/**
	 * Removes one reference from an object in the counter. If the ref count drops to 0 the object is removed from the
	 * counter completely.
	 * 
	 * @param id
	 *            is a unique ID for the object.
	 * @return the new ref count
	 */
	public int removeRef(K id) {
		RefRec rec = mapIdToRec.get(id);
		if (rec == null) {
			return 0;
		}
		int newCount = rec.removeRef();
		if (newCount <= 0) {
			mapIdToRec.remove(id);
		}
		return newCount;
	}

	/**
	 * Returns a complete list of the values in the counter.
	 * 
	 * @return a Collection containing the values.
	 */
	public List<V> values() {
		int size = mapIdToRec.size();
		ArrayList<V> list = new ArrayList<V>(size);
		Iterator<RefRec> iter = mapIdToRec.values().iterator();
		while (iter.hasNext()) {
			RefRec rec = iter.next();
			list.add(rec.getValue());
		}
		return list;
	}
}
