/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import com.jidesoft.docking.DockableFrame;
import com.onaro.client.leekui.ui.IMemento;
import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IViewReference;
import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.PartInitException;
import com.onaro.client.leekui.ui.internal.registry.ViewRegistry;
import com.onaro.client.leekui.ui.part.ViewPart;
import com.onaro.client.leekui.ui.views.IViewDescriptor;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.ViewReference
 */
public final class ViewReference extends WorkbenchPartReference implements IViewReference {

	final ViewFactory viewFactory;

	private String secondaryId;

	ViewPane viewPane;

	private IMemento memento;

	public ViewReference(ViewFactory viewFactory, String id, String secondaryId, IMemento memento) {
		this.viewFactory = viewFactory;

		ViewRegistry viewRegistry = Workbench.getInstance().getViewRegistry();

		IViewDescriptor descriptor = viewRegistry.find(id);

		String title = null;
		Icon icon = null;

		if (descriptor != null) {
			icon = descriptor.getIcon();
			title = descriptor.getLabel();
		}

		this.memento = memento;

		String name = null;

		if (memento != null) {
			name = memento.getString(IWorkbenchConstants.TAG_PART_NAME);
		}

		if (name == null) {
			name = title;
		}

		init(id, title, "", icon, name, ""); //$NON-NLS-1$ //$NON-NLS-2$

		this.secondaryId = secondaryId;

	}

	@Override
	protected PartPane createPane() {

		if (viewPane == null) {
			viewPane = new ViewPane(this, this.viewFactory.page);
		}

		return viewPane;
	}

	public String getSecondaryId() {
		return secondaryId;
	}

	public ViewPart getView(boolean restore) {
		return (ViewPart) getPart(restore);
	}

	@Override
	protected IWorkbenchPart createPart() {
		IViewPart part = null;

		try {
			part = createPartHelper();
		}
		catch (Throwable t) {
			WorkbenchPlugin.log(t);
		}

		if (part == null) {
			// TODO: create an error part

			return null;
		}

		return part;
	}

	private IViewPart createPartHelper() throws PartInitException {
		IViewDescriptor desc = getDescriptor();

		if (desc == null) {
			String msg = null;
			throw new PartInitException(msg);
		}
		
		IMemento stateMem = null;
		if (memento != null) {
			stateMem = memento.getChild(IWorkbenchConstants.TAG_VIEW_STATE);
		}

		IViewPart viewPart;

		try {
			viewPart = desc.createView();
		}
		catch (Throwable t) {
			String msg = null;
			throw new PartInitException(msg, t);
		}

		ViewSite viewSite = new ViewSite(this, viewPart, viewFactory.page, getId(), null, null);

		viewSite.setActionBars(new ViewActionBars(this));

		try {
			viewPart.init(viewSite, stateMem);
		}
		catch (PartInitException e) {
			WorkbenchPlugin.log(e);
			return null;
		}

		final JComponent partComponent = viewPart.createPartComponent();

		Runnable runnable = new Runnable() {
			public void run() {
				((ViewPane) getPane()).setViewComponent(partComponent);
			}
		};

		SwingUtilities.invokeLater(runnable);

		viewPart.addPropertyChangeListener(new ViewPcListener());

		return viewPart;
	}

	public IViewDescriptor getDescriptor() {
		return getDescriptor(getId());
	}

	private IViewDescriptor getDescriptor(String id) {
		return Workbench.getInstance().getViewRegistry().find(id);
	}

	public IMemento getMemento() {
		return memento;
	}

    class ViewPcListener implements PropertyChangeListener {


        public ViewPcListener() {
            //do nothing
        }

        public void propertyChange(PropertyChangeEvent evt) {

            final IViewPart view = (IViewPart) evt.getSource();
            final String propertyName = evt.getPropertyName();

            if (!propertyName.startsWith("title")) return; //$NON-NLS-1$

            final DockableFrame frame = WorkbenchPageUtils.getDockableFrame(view);

            if (frame == null) return;

            Runnable runnable = new Runnable() {
                public void run() {
                    if (propertyName.equals("title")) { //$NON-NLS-1$
                        String title = WorkbenchPageUtils.getTitle(view);
                        frame.setTabTitle(title);
                    } else if (propertyName.equals("titleIcon")) { //$NON-NLS-1$
                        Icon icon = WorkbenchPageUtils.getTitleIcon(view);
                        frame.setFrameIcon(icon);
                    }
                }
            };

            SwingUtilities.invokeLater(runnable);

        }
    }
}

