/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.awt.Component;
import java.awt.Container;
import java.util.concurrent.atomic.AtomicLong;

import javax.swing.Icon;
import javax.swing.SwingUtilities;

import com.jidesoft.docking.DockableFrame;
import com.jidesoft.document.DocumentComponent;
import com.jidesoft.document.DocumentPane;
import com.onaro.client.leekui.ui.IEditorReference;
import com.onaro.client.leekui.ui.IViewPart;
import com.onaro.client.leekui.ui.IViewReference;
import com.onaro.client.leekui.ui.IViewSite;
import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.PartInitException;

public class WorkbenchPageUtils {

	private static WorkbenchPageUtils SINGLETON = new WorkbenchPageUtils();

	private static WorkbenchPageUtils getInstance() {
		return SINGLETON;
	}

	private AtomicLong docNameCounter;

	private WorkbenchPageUtils() {
		docNameCounter = new AtomicLong(0);
	}

	/**
	 * Create the viewPane an asynchronously get the part.
	 */
	public static ViewPane createViewPane(WorkbenchPage page, String id, String secondaryId) throws PartInitException {
		if (page == null) throw new IllegalArgumentException();
		if (id == null) throw new IllegalArgumentException();

		ViewFactory viewFactory = page.getViewFactory();

		final ViewReference viewRef = viewFactory.createView(id, secondaryId);

		return createViewPane(page, viewRef);
	}

	public static ViewPane createViewPane(WorkbenchPage page, final ViewReference viewRef) throws PartInitException {
		if (page == null) throw new IllegalArgumentException();
		if (viewRef == null) throw new IllegalArgumentException();

		ViewPane viewPane = (ViewPane) viewRef.getPane();

		Runnable runnable = new Runnable() {
			public void run() {
				// force creation of the view part component
				viewRef.getPart(true);
			}
		};

		// TODO: do this in the correct threading way
//		Thread t = new Thread(runnable);
//		t.start();
		try {
			Util.invoke(runnable);
		}
		catch(Exception e) {
			throw new PartInitException(e);
		}

		return viewPane;
	}

	public static IEditorReference getEditorReference(DocumentComponent docComponent) {
		if (docComponent == null) throw new IllegalArgumentException();

		EditorPane editorPane = (EditorPane) docComponent.getComponent();

		return editorPane.getEditorReference();
	}

	public static DocumentComponent createDocumentComponent(EditorPane editorPane) throws PartInitException {
		if (editorPane == null) throw new IllegalArgumentException();

		String docName = Long.toHexString(getInstance().docNameCounter.incrementAndGet());

		IWorkbenchPart part = editorPane.getPartReference().getPart(true);

		String docTitle = part.getTitle();

		Icon docIcon = getTitleIcon(part);

		final DocumentComponent docComponent = new DocumentComponent(editorPane, docName, docTitle, docIcon);

		return docComponent;
	}

	public static String getDockableFrameKey(String viewId) {
		return getDockableFrameKey(viewId, null);
	}

	public static String getDockableFrameKey(IViewReference viewRef) {

		return ViewFactory.getKey(viewRef);
	}

	public static String getDockableFrameKey(String viewId, String viewSecondaryId) {

		return ViewFactory.getKey(viewId, viewSecondaryId);
	}

	public static ViewPane getViewPane(ViewReference viewRef) {
		if (viewRef == null) throw new IllegalArgumentException();

		ViewPane viewPane = (ViewPane) viewRef.getPane();

		return viewPane;
	}

	/**
	 * This method may return null.
	 */
	public static ViewPane getViewPane(DockableFrame dockableFrame) {
		
		if (dockableFrame == null) throw new IllegalArgumentException();
		
		Container contentPane = dockableFrame.getContentPane();
		
		ViewPane viewPane = null;
		
		if (contentPane instanceof ViewPane)
		{
			viewPane = (ViewPane) contentPane;
		}
		
		
		return viewPane;
	}
	
	/**
	 * 
	 * @param viewPane (Optional).
	 */
	public static void setViewPane(DockableFrame dockableFrame, ViewPane viewPane)
	{
		if (dockableFrame == null) throw new IllegalArgumentException();
		
		Container contentPane;
		
		if (viewPane == null)
		{
			//just a filler
			contentPane = new Container();
		}
		else
		{
			contentPane = viewPane;
		}
		
		//can't set content pane to null
		dockableFrame.setContentPane(contentPane);
	}

	public static String getDockableFrameKey(IViewPart viewPart) {

		if (viewPart == null) return null;

		IViewSite viteSite = viewPart.getViewSite();

		String id = viteSite.getId();
		String secondaryId = viteSite.getSecondaryId();

		return getDockableFrameKey(id, secondaryId);
	}

	/**
	 * This may return <code>null</code>.
	 */
	public static IViewReference getViewReference(DockableFrame dockableFrame) {
		if (dockableFrame == null) return null;

		ViewPane viewPane = getViewPane(dockableFrame);
		
		if (viewPane == null) return null;

		return viewPane.getViewReference();
	}

	public static DockableFrame getDockableFrame(IViewPart view) {

		ViewSite viewSite = (ViewSite) view.getSite();

		if (viewSite == null) return null;

		ViewReference viewRef = (ViewReference) viewSite.getPartReference();

		if (viewRef == null) return null;

		ViewPane viewPane = (ViewPane) viewRef.getPane();

		if (viewPane == null) return null;

		return getDockableFrame(viewPane);
	}

	public static Icon getTitleIcon(IWorkbenchPart part) {
		Icon titleIcon = part.getTitleIcon();

		if (titleIcon != null) return titleIcon;

		PartSite partSite = (PartSite) part.getSite();

		WorkbenchPartReference partRef = (WorkbenchPartReference) partSite.getPartReference();

		return partRef.getTitleIcon();
	}

	public static String getTitle(IViewPart viewPart) {
		String title = viewPart.getTitle();

		if (title != null && title.length() > 0) return title;

		ViewSite viewSite = (ViewSite) viewPart.getSite();

		ViewReference viewRef = (ViewReference) viewSite.getPartReference();

		return viewRef.getTitle();
	}

	public static DockableFrame getDockableFrame(Component descendant) {
		DockableFrame dockableFrame = getAncestorOfClass(DockableFrame.class, descendant);

		if (dockableFrame != null) return null;

		// TODO: what if they click on the jide tabbed pane owning a dockable frame

		return null;
	}

	public static DocumentPane getDocumentPane(Component descendant) {
		DocumentPane docPane = getAncestorOfClass(DocumentPane.class, descendant);

		return docPane;
	}

	public static PartPane getPartPane(Component descendant) {
		PartPane partPane = getAncestorOfClass(PartPane.class, descendant);

		return partPane;
	}

	public static <T extends Component> T getAncestorOfClass(Class<T> clazz, Component c) {
		if (clazz == null) throw new IllegalArgumentException();

		if (c == null) return null;

		Component ancestor = SwingUtilities.getAncestorOfClass(clazz, c);

		return clazz.cast(ancestor);
	}

}
