/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Icon;

import com.onaro.client.leekui.ui.IWorkbenchPart;
import com.onaro.client.leekui.ui.IWorkbenchPartReference;
import com.onaro.client.leekui.ui.IWorkbenchPartSite;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.WorkbenchPartReference
 */
public abstract class WorkbenchPartReference implements IWorkbenchPartReference {
	private String id;

	private IWorkbenchPart part;

	private String partName;

	private String contentDescription;

	private String title;

	private String toolTip;

	private Icon defaultIcon = null;

	private Icon icon = null;

	private PropertyChangeSupport propertyChangeSupport;

	protected PartPane pane;

	public final String getId() {
		if (part != null) {
			IWorkbenchPartSite site = part.getSite();
			if (site != null) {
				return site.getId();
			}
		}
		return id;
	}

	public final void addPropertyChangeListener(PropertyChangeListener pcListener) {
		getPropertyChangeSupport().addPropertyChangeListener(pcListener);
	}

	public final void removePropertyChangeListener(PropertyChangeListener pcListener) {
		getPropertyChangeSupport().removePropertyChangeListener(pcListener);
	}

	protected final PropertyChangeSupport getPropertyChangeSupport() {
		if (propertyChangeSupport == null) {
			propertyChangeSupport = new PropertyChangeSupport(this);
		}

		return propertyChangeSupport;
	}

	protected final void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		getPropertyChangeSupport().firePropertyChange(propertyName, oldValue, newValue);
	}

	public final void dispose() {
		if (isDisposed()) {
			return;
		}

		propertyChangeSupport = null;
	}

	public boolean isDisposed() {
		return false;
	}

	public final IWorkbenchPart getPart(boolean restore) {

		if (isDisposed()) {
			return null;
		}

		if (part == null && restore) {

			synchronized (this) {
				
				//someone else may have created it while i was waiting on sync
				if (part != null) return part;
				
				IWorkbenchPart newPart = createPart();

				if (newPart != null) {
					part = newPart;

					PartPane pane = getPane();
					WorkbenchPage page = pane.getPage();
					final WorkbenchPage.WorkbenchPageComponent pageComponent = page.getComponent();

					Runnable runnable = new Runnable() {
						public void run() {
							pageComponent.validate();
						}
					};
					Util.invokeSafely(runnable);
//					SwingUtilities.invokeLater(runnable);
				}
			}
		}

		return part;
	}

	public final PartPane getPane() {

		if (pane == null) {

			pane = createPane();
		}

		return pane;
	}

	protected abstract IWorkbenchPart createPart();

	protected abstract PartPane createPane();

	public void init(String id, String title, String toolTip, Icon icon, String paneName, String contentDescription) {

		this.id = id;
		this.title = title;
		this.toolTip = toolTip;
		this.partName = paneName;
		this.contentDescription = contentDescription;
		this.defaultIcon = icon;

		this.icon = computeIcon();
	}

	protected Icon computeIcon() {
		if (part != null) {
			// return ImageDescriptor.createFromImage(part.getTitleImage(), Display.getCurrent());
		}

		return defaultIcon;
	}

	protected void refreshFromPart() {

		// setPartName(computePartName());
		setTitle(computeTitle());
		setContentDescription(computeContentDescription());
		setToolTip(getRawToolTip());
		setIcon(computeIcon());
	}

	protected void setContentDescription(String newContentDescription) {
		String oldContentDescription = this.contentDescription;

		contentDescription = newContentDescription;

		this.firePropertyChange("contentDescription", oldContentDescription, newContentDescription); //$NON-NLS-1$
	}

	protected void setIcon(Icon newIcon) {
		Icon oldIcon = this.icon;

		this.icon = newIcon;

		this.firePropertyChange("image", oldIcon, newIcon); //$NON-NLS-1$
	}

	protected void setToolTip(String newToolTip) {
		String oldToolTip = this.toolTip;

		toolTip = newToolTip;

		firePropertyChange("toolTip", oldToolTip, toolTip); //$NON-NLS-1$
	}

	protected String computePartName() {
		return getRawPartName();
	}

	protected String computeTitle() {
		return getRawTitle();
	}

	protected final String getRawTitle() {
		return Util.safeString(part.getTitle());
	}

	protected final String getRawToolTip() {
		return Util.safeString(part.getTitleToolTip());
	}

	protected final String getRawPartName() {
		return Util.safeString(part.getPartName());
	}

	protected void setPartName(String newPartName) {

		String oldPartName = this.partName;

		partName = newPartName;

		firePropertyChange("partName", oldPartName, newPartName); //$NON-NLS-1$
	}

	protected void setTitle(String newTitle) {

		String oldTitle = title;

		title = newTitle;

		firePropertyChange("title", oldTitle, newTitle); //$NON-NLS-1$
	}

	protected String computeContentDescription() {
		return getRawContentDescription();
	}

	protected final String getRawContentDescription() {
		return part.getContentDescription();
	}

	public String getPartName() {
		return Util.safeString(partName);
	}

	public String getTitle() {
		return Util.safeString(title);
	}

	public final Icon getTitleIcon() {
		return icon;
	}
}
