/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.dialogs;

import com.onaro.client.leekui.ui.internal.Workbench;
import com.onaro.client.leekui.ui.internal.registry.Category;
import com.onaro.client.leekui.ui.internal.registry.ViewRegistry;
import com.onaro.client.leekui.ui.views.IViewCategory;
import com.onaro.client.leekui.ui.views.IViewDescriptor;
import org.apache.commons.lang.ArrayUtils;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Comparator;

public class ViewRegistryTree extends JTree {

	private Dimension preferredScrollableViewportSize;
	
	public static TreeModel createModel() { 
		return createModel(null);
	}
	
	/**
	 * 
	 * @param ancestorCategoryId (Optional) Only show descendent views of this categoryId.
	 */
	public static TreeModel createModel(String ancestorCategoryId) {

		DefaultMutableTreeNode rootTreeNode = new DefaultMutableTreeNode();

		ViewRegistry viewRegistry = Workbench.getInstance().getViewRegistry();

		Map<String,ViewCategoryTreeNode> categoryTreeNodesById = new HashMap<String,ViewCategoryTreeNode>();

        for (IViewCategory category : viewRegistry.getCategories()) {

			String categoryId = category.getId();
			
			if (ancestorCategoryId != null) {
				String[] categoryPathSegments = category.getPath().segments();
				boolean isDescendent = ancestorCategoryId.equals(categoryId) || ArrayUtils.contains(categoryPathSegments, ancestorCategoryId);
				if (!isDescendent) continue; 
			}
			
			ViewCategoryTreeNode categoryTreeNode = new ViewCategoryTreeNode(category);
			rootTreeNode.add(categoryTreeNode);

			categoryTreeNodesById.put(categoryId, categoryTreeNode);
		}

        List<IViewDescriptor> views = viewRegistry.getViews();

        //sort the views alphbetically
        Collections.sort(views, new Comparator<IViewDescriptor>() {
            public int compare(IViewDescriptor view1, IViewDescriptor view2) {
                String view1Label = view1.getLabel() != null ? view1.getLabel() : ""; //$NON-NLS-1$
                String view2Label = view2.getLabel() != null ? view2.getLabel() : ""; //$NON-NLS-1$
                return view1Label.compareTo(view2Label);
            }
        });

        for (IViewDescriptor viewDescriptor : views) {
			List<String> categoryPath = viewDescriptor.getCategoryPath();

			Category<IViewDescriptor> miscCategory = viewRegistry.getMiscCategory();

			String categoryId = (miscCategory != null) ? miscCategory.getId() : null;

			if (categoryPath != null && !categoryPath.isEmpty()) {
				categoryId = categoryPath.get(0);
			}

			ViewCategoryTreeNode categoryTreeNode = categoryTreeNodesById.get(categoryId);

			if (categoryTreeNode == null) continue;
			
			categoryTreeNode.add(new ViewDescriptorTreeNode(viewDescriptor));
		}

		return new DefaultTreeModel(rootTreeNode);
	}

	private static final long serialVersionUID = 1L;

	public ViewRegistryTree(TreeModel treeModel) {

		super(treeModel);

		initialize();
	}
	
	private IViewDescriptor getViewDescriptor(TreePath treePath)
	{
		if (treePath == null) return null;

		Object lastPathComponent = treePath.getLastPathComponent();

		if (lastPathComponent instanceof ViewDescriptorTreeNode) {
			ViewDescriptorTreeNode vdTreeNode = (ViewDescriptorTreeNode) lastPathComponent;

			return vdTreeNode.getUserObject();
		}

		return null;
	}
	
	public List<IViewDescriptor> getSelectedViewDescriptors()
	{
		TreePath[] selectionPaths = this.getSelectionPaths();
		
		if (selectionPaths == null || selectionPaths.length < 1) return Collections.emptyList();
		
		List<IViewDescriptor> views = new ArrayList<IViewDescriptor>();
		
		
		for (TreePath treePath : selectionPaths)
		{
			IViewDescriptor view = getViewDescriptor(treePath);
			
			if (view == null) continue;
			
			views.add(view);
		}
		
		return views;
	}

	public IViewDescriptor getSelectedViewDescriptor() {
		TreePath selectionPath = this.getSelectionPath();

		return getViewDescriptor(selectionPath);
	}

	private void initialize() {
		this.setCellRenderer(new ViewRegistryTreeCellRenderer());

		this.setRootVisible(false);
		this.setShowsRootHandles(true);
	}

	@Override
	public Dimension getPreferredScrollableViewportSize() {

		if (preferredScrollableViewportSize == null) {

			// these values were copied from eclipse
			preferredScrollableViewportSize = new Dimension(230, 270);
		}

		return preferredScrollableViewportSize;
	}
}

class ViewRegistryTreeCellRenderer extends DefaultTreeCellRenderer {
	private static final long serialVersionUID = 1L;

	@Override
	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded,
			boolean leaf, int row, boolean hasFocus) {
		Component comp = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);

		if (value instanceof ViewDescriptorTreeNode) {
			Icon icon = ((ViewDescriptorTreeNode) value).getUserObject().getIcon();

			setIcon(icon);
		}

		return comp;
	}
}

class ViewCategoryTreeNode extends DefaultMutableTreeNode {

	private static final long serialVersionUID = 1L;

	public ViewCategoryTreeNode(IViewCategory viewCategory) {
		super(viewCategory);
	}

	@Override
	public IViewCategory getUserObject() {
		return (IViewCategory) super.getUserObject();
	}

	@Override
	public String toString() {
		return getUserObject().getLabel();
	}
}

class ViewDescriptorTreeNode extends DefaultMutableTreeNode {
	private static final long serialVersionUID = 1L;

	public ViewDescriptorTreeNode(IViewDescriptor viewDescriptor) {
		super(viewDescriptor);
	}

	@Override
	public IViewDescriptor getUserObject() {
		return (IViewDescriptor) super.getUserObject();
	}

	@Override
	public String toString() {
		return getUserObject().getLabel();
	}
}
