/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.progress;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JToggleButton;
import javax.swing.Timer;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;

public class ProgressMonitorComponentManager implements IProgressMonitor {
	
	static final Logger logger = Logger.getLogger(ProgressMonitorComponentManager.class);

	// Delay before a task is added to the progress dialog
	private static final int TIMER_DELAY = 1000;
	
	protected final ProgressMonitorsDialogManager dialogManager;
	
	protected final Object componentMutex;
	
	protected ProgressMonitorComponent component;
	
	protected final AtomicBoolean beganTask;
	
	protected String taskName = null;
	
	protected String subTask = null;
	
	protected boolean canceled = false;
	
	protected final boolean cancelable;
	
	protected final boolean background;
	
	protected final Object timerMutex;
	
	protected Timer timer;
	
	public ProgressMonitorComponentManager(ProgressMonitorsDialogManager manager, boolean cancelable, boolean background) {
		if (manager == null) throw new IllegalArgumentException("manager"); //$NON-NLS-1$
		
		this.dialogManager = manager;
		this.cancelable = cancelable;
		this.background = background;
		
		beganTask = new AtomicBoolean(false);
		
		componentMutex = new Object();
		timerMutex = new Object();
	}
	
	private Timer getTimer() {
		return getTimer(true);
	}
	
	private Timer getTimer(boolean create) {
		if (timer == null && create) {
			synchronized (timerMutex) {
				if (timer == null) {
					timer = new Timer(TIMER_DELAY, new AddComponentActionListener());
					timer.setRepeats(false);
				}
			}
		}
		
		return timer;
	}
	
	public ProgressMonitorComponent getComponent() {
		return getComponent(true);
	}
	
	public ProgressMonitorComponent getComponent(boolean create) {
		if (component == null && create) {
			synchronized (componentMutex) {
				if (component == null) {
					component = createComponent();
				}
			}
			
		}
		
		return component;
	}
	
	private void markCanceled(JToggleButton button) {
		button.setSelected(true);
		button.setEnabled(false);
		button.setFocusable(false);
	}
	
	private ProgressMonitorComponent createComponent() {
		ProgressMonitorComponent progressMonitorComponent = new ProgressMonitorComponent();
		JToggleButton button = progressMonitorComponent.getButton();
		button.setEnabled(cancelable);
		if (canceled) {
			markCanceled(button);
		}
		button.addActionListener(new ButtonActionListener());
		return progressMonitorComponent;
	}

	public void beginTask(String name, int totalWork) {
		if (beganTask.compareAndSet(true, true)) throw new IllegalStateException("Already began task"); //$NON-NLS-1$
				
		ProgressMonitorComponent progressMonitorComponent = getComponent();
		JProgressBar progressBar = progressMonitorComponent.getProgressBar();
		
		
		
		boolean isTotalWorkKnown = totalWork != IProgressMonitor.UNKNOWN;
		progressBar.setIndeterminate(!isTotalWorkKnown);
		
		if (isTotalWorkKnown) {
			progressBar.setMinimum(0);
			progressBar.setValue(0);
			progressBar.setMaximum(totalWork);	
		} 
		
		setTaskName(name);
		subTask(null);
		
		if (logger.isDebugEnabled()) {
			logger.debug("Beginning task: " + name); //$NON-NLS-1$
		}
		
		getTimer().start();
	}
	
	protected final ProgressMonitorsDialogManager getDialogManager() {
		return dialogManager;
	}

	public void done() {
		
		if (logger.isDebugEnabled()) {
			logger.debug("Done with task: " + getTaskName()); //$NON-NLS-1$
		}
		
		Timer timer = getTimer(false);
		if (timer != null) {
			timer.stop();
		}
		
		ProgressMonitorComponent progressMonitorComponent = getComponent(false);
		if (progressMonitorComponent != null) {
			getDialogManager().remove(progressMonitorComponent);
		}
	}

	public void internalWorked(double work) {
		//do nothing
	}
	
	public boolean isBackground() {
		return background;
	}

	public boolean isCanceled() {
		return canceled;
	}	

	public void setCanceled(boolean value) {
		canceled = value;
		
		if (value == true && logger.isDebugEnabled()) {
			logger.debug("Canceling task: " + getTaskName()); //$NON-NLS-1$
		}
		
		synchronized (componentMutex) {
			ProgressMonitorComponent component = getComponent(false);
			if (component != null) {
				markCanceled(component.getButton());
			}
		}
	}

	public void setTaskName(String name) {
		this.taskName = name;
		JLabel northLabel = getComponent().getNorthLabel();
		northLabel.setText(name);
	}
	
	public String getTaskName() {
		return taskName;
	}

	public void subTask(String name) {
		this.subTask = name;
		JLabel southLabel = getComponent().getSouthLabel();
		southLabel.setText(name);
	}
	
	public String getSubTask() {
		return subTask;
	}

	public void worked(int work) {
		synchronized (componentMutex) {
			JProgressBar progressBar = getComponent().getProgressBar();
			int value = progressBar.getValue();
			value += work;
			progressBar.setValue(value);
		}
	}
	
	class ButtonActionListener implements ActionListener {

		public ButtonActionListener() {
			//do nothing
		}
		
		public void actionPerformed(ActionEvent event) {
			setCanceled(true);
		}
		
	}
	
	class AddComponentActionListener implements ActionListener {

		public AddComponentActionListener() {
			//do nothing
		}
		
		public void actionPerformed(ActionEvent event) {
			ProgressMonitorComponent progressMonitorComponent = getComponent();
			getDialogManager().add(progressMonitorComponent, background);
		}
		
	}

}
