/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.progress;

import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JProgressBar;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;

import com.jidesoft.status.StatusBarItem;
import com.onaro.client.leekui.jface.preferences.Messages;
import com.onaro.client.leekui.ui.internal.Util;

public class ProgressMonitorsDialogManager {
	
	static final Logger logger = Logger.getLogger(ProgressMonitorsDialogManager.class);

	/**
	 * Auto close the dialog when there are not ProgressMonitorComponents to show.
	 */
	private boolean autoCloseDialog;
		
	private boolean manuallyShowing;

	private final Object dialogMutex;

	private ProgressMonitorsDialog dialog;
	
	private MyProgressBarItem statusBarItem;
	
	public ProgressMonitorsDialogManager() {
		dialogMutex = new Object();
		autoCloseDialog = true;
		manuallyShowing = false;
	}

	protected final ProgressMonitorsDialog getDialog() {
		if (dialog == null) {
			synchronized (dialogMutex) {
				if (dialog == null) {
					dialog = new ProgressMonitorsDialog(null);
					dialog.addWindowListener(new WindowAdapter() {
						@Override
						public void windowClosing(WindowEvent e) {
							manuallyShowing = false;
						}
					});
				}
			}
		}

		return dialog;
	}

	public IProgressMonitor createProgressMonitor(boolean cancelable, boolean background) {
		ProgressMonitorComponentManager progressMonitorComponentController = new ProgressMonitorComponentManager(
				this, cancelable, background);
		return progressMonitorComponentController;
	}

	public void add(final ProgressMonitorComponent progressMonitorComponent, final boolean background) {
		try {
			Util.invoke(new Runnable() {
				public void run() {
					ProgressMonitorsDialog dialog = getDialog();
					dialog.add(progressMonitorComponent);
					if (!background) {
						show(dialog);
					}
					updateProgressStatusBarItem();
				}
			});
		}
		catch (Exception e) {
			logger.warn("Failed to add progress item" + " - " + e.getMessage() , e); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public void showDialog() {
		manuallyShowing = true;
		show(getDialog());
		getDialog().toFront();
	}
	
	private void show(ProgressMonitorsDialog dialog) {
		if (dialog.isVisible()) return;
		
		Window parent = Util.getActiveWindow();
		dialog.setLocationRelativeTo(parent);
		Point location = dialog.getLocation();
		dialog.setLocation(location.x, Math.max(location.y - dialog.getHeight(), 0));		
		
		dialog.setVisible(true);
	}

	public void remove(final ProgressMonitorComponent progressMonitorComponent) {
		try {
            Util.invoke(new Runnable() {
				public void run() {
					ProgressMonitorsDialog dialog = getDialog();
					dialog.remove(progressMonitorComponent);
					if (!manuallyShowing && autoCloseDialog && dialog.getProgressMonitorComponentCount() < 1) {
						dialog.setVisible(false);
					}
					updateProgressStatusBarItem();
				}
			});
		}
		catch (Exception e) {
			logger.warn("Failed to remove progress item" + " - " + e.getMessage(), e); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	private void updateProgressStatusBarItem() {
		boolean inProgress = dialog.getProgressMonitorComponentCount() > 0;
		getProgressStatusBarItem().showProgress(inProgress);
	}
	
	public MyProgressBarItem getProgressStatusBarItem() {
		if (statusBarItem == null) {
			statusBarItem = new MyProgressBarItem();
			statusBarItem.addMouseListener(new MouseAdapter() {
				private void processClick(MouseEvent e) {
					if (e.getClickCount() == 2) {
						showDialog();
					}
				}

				@Override
				public void mouseClicked(MouseEvent e) {
					processClick(e);
				}

				@Override
				public void mousePressed(MouseEvent e) {
					processClick(e);
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					processClick(e);
				}
			});
		}
		return statusBarItem;
	}
	
	private class MyProgressBarItem extends StatusBarItem {
		private static final long serialVersionUID = 1L;
		
		private JProgressBar progressBar;
		
		public MyProgressBarItem() {
			initialize();
		}
		
		private void initialize() {
			setLayout(new GridLayout());
			progressBar = new JProgressBar();
			progressBar.setName("statusProgressBar"); //$NON-NLS-1$
			progressBar.setIndeterminate(true);
			setPreferredWidth(55);
			progressBar.setVisible(false);
			add(progressBar);
		}
		
		public void showProgress(boolean inProgress) {
			progressBar.setVisible(inProgress);
		}

		@Override
		public String getItemName() {
			return Messages.INSTANCE.getProgressItem();
		}
		
	}
}
