/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.registry;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.onaro.client.leekui.jface.preferences.Messages;

/**
 * Category provides for hierarchical grouping of elements registered in the registry. One extension normally defines a
 * category, and other reference it via its ID.
 * <p>
 * A category may specify its parent category in order to achieve hierarchy.
 * </p>
 * 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.registry.Category
 */
public class Category<T> {
	/**
	 * Name of the miscellaneous category
	 */
	public final static String MISC_NAME = Messages.INSTANCE.getGeneralCategory();

	/**
	 * Identifier of the miscellaneous category
	 */
	public final static String MISC_ID = "com.onaro.client.leekui.ui.internal.otherCategory"; //$NON-NLS-1$

	private String id;

	private String name;

	private List<String> parentPath;

	private ArrayList<T> elements;

	private String rawParentPath;

	public Category() {
		this(MISC_ID, MISC_NAME);
	}

	public void clear() {
		if (elements != null) {
			elements.clear();
		}
	}

	public Category(String id, String name) {
		this(id, name, null);
	}

	public Category(String id, String name, String rawParentPath) {
		this.id = id;
		this.name = name;
		this.rawParentPath = rawParentPath;
	}

	/**
	 * Return the elements contained in this category.
	 * 
	 * @return the elements
	 */
	public ArrayList<T> getElements() {
		return elements;
	}

	/**
	 * Return whether this category has child elements.
	 * 
	 * @return whether this category has child elements
	 */
	public boolean hasElements() {
		if (elements != null) {
			return !elements.isEmpty();
		}

		return false;
	}

	/**
	 * Return the unparsed parent path. May be <code>null</code>.
	 * 
	 * @return the unparsed parent path or <code>null</code>
	 */
	public String getRawParentPath() {
		return rawParentPath;
	}

	public String getId() {
		return id;
	}

	public String getLabel() {
		return name;
	}

	public List<String> getParentPath() {
		if (parentPath != null) {
			return parentPath;
		}

		String unparsedPath = getRawParentPath();
		if (unparsedPath != null) {
			parentPath = Arrays.asList(unparsedPath.split("/")); //$NON-NLS-1$
		}

		return parentPath;
	}

	public boolean hasElement(T o) {
		if (elements == null) {
			return false;
		}
		if (elements.isEmpty()) {
			return false;
		}
		return elements.contains(o);
	}

	public void addElement(T element) {
		if (elements == null) {
			elements = new ArrayList<T>(5);
		}
		elements.add(element);
	}
}
