/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.ui.internal.registry;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

import com.onaro.client.leekui.ui.views.IViewCategory;
import com.onaro.client.leekui.ui.views.IViewDescriptor;
import com.onaro.client.leekui.ui.views.IViewRegistry;

/** 
 * This class is a reimplementation of the Eclipse class org.eclipse.ui.internal.registry.ViewRegistry
 */
public class ViewRegistry implements IViewRegistry {
	private final Map<String,IViewDescriptor> viewDescriptorsByIdMap;

	private final Map<String,Category<IViewDescriptor>> categoriesByIdMap;

	private Category<IViewDescriptor> miscCategory;

	private boolean dirtyViewCategoryMappings = true;

	public ViewRegistry() {
		viewDescriptorsByIdMap = new HashMap<String,IViewDescriptor>();
		categoriesByIdMap = new HashMap<String,Category<IViewDescriptor>>();
	}

	public void createCategory(String id, String label, String rawParentPath) {
		if (id == null || id.length() < 1) throw new IllegalArgumentException();
		if (label == null || label.length() < 1) throw new IllegalArgumentException();

		Category<IViewDescriptor> category = new Category<IViewDescriptor>(id, label, rawParentPath);

		add(category);
	}

	public void add(Category<IViewDescriptor> category) {
		if (category == null) throw new IllegalArgumentException();

		String categoryId = category.getId();

		if (categoryId == null) throw new IllegalArgumentException();

		categoriesByIdMap.put(categoryId, category);

		dirtyViewCategoryMappings = true;
	}

	public void add(IViewDescriptor viewDescriptor) {

		if (viewDescriptor == null) throw new IllegalArgumentException();

		String viewId = viewDescriptor.getId();

		if (viewId == null) throw new IllegalArgumentException();

		viewDescriptorsByIdMap.put(viewId, viewDescriptor);

		dirtyViewCategoryMappings = true;
	}

	public IViewDescriptor find(String id) {
		return viewDescriptorsByIdMap.get(id);
	}

	public List<IViewDescriptor> getViews() {

		return new ArrayList<IViewDescriptor>(viewDescriptorsByIdMap.values());
	}

	/**
	 * Adds each view in the registry to a particular category. The view category may be defined in xml. If not, the
	 * view is added to the "misc" category.
	 */
	public void mapViewsToCategories() {
		if (dirtyViewCategoryMappings) {
			dirtyViewCategoryMappings = false;
			// clear all category mappings
			for (Category<IViewDescriptor> category : categoriesByIdMap.values()) {
				category.clear(); // this is bad
			}

			if (miscCategory != null) {
				miscCategory.clear();
			}

			for (IViewDescriptor desc : viewDescriptorsByIdMap.values()) {
				Category<IViewDescriptor> cat = null;
				List<String> catPath = desc.getCategoryPath();
				if (catPath != null && !catPath.isEmpty()) {
					String rootCat = catPath.get(0);
					cat = internalFindCategory(rootCat);
				}
				if (cat != null) {
					if (!cat.hasElement(desc)) {
						cat.addElement(desc);
					}
				}
				else {
					if (miscCategory == null) {
						miscCategory = new Category<IViewDescriptor>();
						add(miscCategory);
					}
					if (catPath != null) {
						// If we get here, this view specified a category which
						// does not exist. Add this view to the 'Other' category
						// but give out a message (to the log only) indicating
						// this has been done.
						// String fmt = "Category {0} not found for view {1}. This view added to ''{2}'' category.";
						// //$NON-NLS-1$
						// WorkbenchPlugin.log(MessageFormat
						// .format(fmt, new Object[] { catPath[0],
						// desc.getId(), miscCategory.getLabel() }));
					}
					miscCategory.addElement(desc);
				}
			}
		}
	}
	
	/**
     * Returns the Misc category. This may be <code>null</code> if there are
     * no miscellaneous views.
     * 
     * @return the misc category or <code>null</code>
     */
    public Category<IViewDescriptor> getMiscCategory() {
        return miscCategory;
    }

	public List<IViewCategory> getCategories() {
		mapViewsToCategories();

		List<IViewCategory> list = new ArrayList<IViewCategory>();
		
		for (Category<IViewDescriptor> category : categoriesByIdMap.values())
		{
			list.add(new ViewCategoryProxy(category));
		}
		
		return list;
	}

	/**
	 * Returns the category with no updating of the view/category mappings.
	 * 
	 * @param id
	 *            the category id
	 * @return the Category
	 * 
	 */
	private Category<IViewDescriptor> internalFindCategory(String id) {
		return categoriesByIdMap.get(id);
	}

	class ViewCategoryProxy implements IViewCategory {
		private Category<IViewDescriptor> rawCategory;

		public ViewCategoryProxy(Category<IViewDescriptor> rawCategory) {
			this.rawCategory = rawCategory;
		}

		public List<IViewDescriptor> getViews() {
			return rawCategory.getElements();
		}

		public String getId() {
			return rawCategory.getId();
		}

		public String getLabel() {
			return rawCategory.getLabel();
		}

		public IPath getPath() {
			String rawParentPath = rawCategory.getRawParentPath();
			if (rawParentPath == null) {
				return new Path(""); //$NON-NLS-1$
			}
			return new Path(rawParentPath);
		}

	};
}
