/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.preferences;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.beans.ExceptionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.prefs.Preferences;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.eclipse.core.runtime.ISafeRunnable;

import com.jidesoft.dialog.ButtonPanel;
import com.jidesoft.tree.TreeUtils;
import com.onaro.client.leekui.jface.dialogs.IMessageConsumer;
import com.onaro.client.leekui.jface.dialogs.IMessageProvider;
import com.onaro.client.leekui.jface.util.DialogUtils;
import com.onaro.client.leekui.jface.util.SafeRunnable;
import com.onaro.client.leekui.ui.PlatformUI;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.preferences.PreferenceDialog
 */
public class PreferenceDialog extends JFrame implements IPreferencePageContainer, IMessageConsumer {

	private static final long serialVersionUID = 1L;

	private JPanel jContentPane = null;
	
	private IPreferenceNode currentNode = null;

	private ButtonPanel buttonPanel = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private JScrollPane jScrollPane = null;

	private JTree nodeTree = null;

	private JPanel centerPanel = null;

	private JPanel pageHeaderPanel = null;

	private JLabel pageTitleLabel = null;

	private JLabel pageDescriptionLabel = null;

	private JSeparator jSeparator = null;

	private PreferenceDialog.PreferenceTreeModel preferenceTreeModel = null;  //  @jve:decl-index=0:visual-constraint="598,174"

	private PreferenceDialog.PreferenceNodeTreeCellRenderer preferenceNodeTreeCellRenderer = null;  //  @jve:decl-index=0:visual-constraint="614,268"

	private JPanel pageContentsPanel = null;
	
	//The id of the last page that was selected
	private static String lastPreferenceId = null;  //  @jve:decl-index=0:

	private CardLayout cardLayout = null;  //  @jve:decl-index=0:visual-constraint="640,103"

	private JSplitPane splitPane = null;
	
	private Preferences preferenceStore = null;
	
	private ExceptionListener exceptionListener = null;

	/**
	 * @param owner
	 */
	public PreferenceDialog() {
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(640, 480);
		this.setTitle(Messages.INSTANCE.getDefaultSettingsTitle());
		this.setContentPane(getJContentPane());
		this.setName("PreferenceDialog"); //$NON-NLS-1$
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getButtonPanel(), BorderLayout.SOUTH);
			jContentPane.add(getSplitPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}
	
	protected boolean showPage(final IPreferenceNode node) {
		if (node == null) {
			selectNodeInTree(currentNode);
			return false;
		}
		// Create the page if necessary
		if (node.getPage() == null) {
			node.createPage();
		}
		IPreferencePage newPage = node.getPage();
		if (newPage == null) {
			selectNodeInTree(currentNode);
			return false;
		}
				
		if (node != currentNode) {
			IPreferencePage currentPage = currentNode == null ? null : currentNode.getPage();
			
			
			if (currentPage != null) {
				if (!currentPage.okToLeave()) {
					JOptionPane.showMessageDialog(this, 
							Messages.INSTANCE.getFailedToLeavePageErrorMessage(), 
							Messages.INSTANCE.getFailedToLeavePageErrorTitle(), 
							JOptionPane.ERROR_MESSAGE);
					selectNodeInTree(currentNode);
					return false;
				}
				currentPage.setVisible(false);
			}
			currentNode = node;
			currentPage = newPage;
			currentPage.setContainer(this);
			
			// Ensure that the page control has been created
			// (this allows lazy page control creation)
			if (currentPage.getComponent(false) == null) {
				final boolean[] failed = { false };
				SafeRunnable.run(new ISafeRunnable() {
					public void handleException(Throwable e) {
						failed[0] = true;
					}
	
					public void run() {
						Component newPage = node.getPage().getComponent(true);
						getPageContentsPanel().add(newPage, node.getId());
					}
				});
				if (failed[0]) {
					selectNodeInTree(currentNode);
					return false;
				}
			}
			
			Component pageComponent = currentPage.getComponent(false);
			
			if (!getPageContentsPanel().isAncestorOf(pageComponent)) {
				getPageContentsPanel().add(pageComponent, node.getId());
			}
			
			currentPage.setVisible(true);
			getCardLayout().show(getPageContentsPanel(), node.getId());
			updateTitle();
			updateMessage();
		} else {
			// If we're being asked to show the current page, just call setVisible(true) on it.
			// This is necessary because we may be opening the dialog and showing the page that was
			// shown when it was last closed. (Bug 11305)
			newPage.setVisible(true);
		}
		// Update buttons anyway since we disabled the Ok button each time
		// we exit the dialog.
		updateButtons();
		
		return true;
	}
	
	/**
	 * Find the <code>IPreferenceNode</code> that has data the same id as the
	 * supplied value.
	 * 
	 * @param nodeId
	 *            the id to search for.
	 * @return <code>IPreferenceNode</code> or <code>null</code> if not
	 *         found.
	 */
	protected IPreferenceNode findNodeMatching(String nodeId) {
		List<IPreferenceNode> nodes = getPreferenceManager().getElements(PreferenceManager.TraversalOrder.POST_ORDER);
		for (IPreferenceNode node : nodes) {
			if (node.getId().equals(nodeId)) {
				return node;
			}
		}
		return null;
	}
	
	private static PreferenceManager getPreferenceManager() {
		return PlatformUI.getWorkbench().getPreferenceManager();
	}
	
	protected void okPressed() {
		SafeRunnable.run(new SafeRunnable() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.core.runtime.ISafeRunnable#run()
			 */
			public void run() {
				getOkButton().setEnabled(false);
				boolean hasFailedOK = false;
				try {
					// Notify all the pages and give them a chance to abort
					List<IPreferenceNode> preferenceNodes = getPreferenceManager().getElements(PreferenceManager.TraversalOrder.PRE_ORDER);
					for (IPreferenceNode node : preferenceNodes) {
						IPreferencePage page = node.getPage();
						if (page != null) {
							if (!page.performOk()){
								hasFailedOK = true;
								return;
							}
						}
					}
				} catch (Exception e) {
					handleException(e);
					hasFailedOK = true;
				} finally {
					//Don't bother closing if the OK failed
					if(hasFailedOK){
						getOkButton().setEnabled(true);
						return;
					}
					setVisible(false);
				}
			}

			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.core.runtime.ISafeRunnable#handleException(java.lang.Throwable)
			 */
			public void handleException(Throwable e) {
				
				clearSelectedNode();
				
				if (getExceptionListener() == null || !(e instanceof Exception)) {
					// log something
					JOptionPane.showMessageDialog(PreferenceDialog.this, 
							Messages.INSTANCE.getFailedToSaveSettingsErrorMessage(), 
							Messages.INSTANCE.getFailedToSaveSettingsErrorTitle(), 
							JOptionPane.ERROR_MESSAGE);
				} else {
					getExceptionListener().exceptionThrown((Exception)e);
				}
			}
		});
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		// Inform all pages that we are cancelling
		List<IPreferenceNode> preferenceNodes = getPreferenceManager().getElements(PreferenceManager.TraversalOrder.PRE_ORDER);
		for (final IPreferenceNode node : preferenceNodes) {
			if (node.getPage() != null) {
				SafeRunnable.run(new SafeRunnable() {
					public void run() {
						if (!node.getPage().performCancel()) {
							return;
						}
					}
				});
			}
		}
		setVisible(false);
	}
	
	/**
	 * Sets the name of the selected item preference. Public equivalent to
	 * <code>setSelectedNodePreference</code>.
	 * 
	 * @param pageId
	 *            The identifier for the page
	 * @since 3.0
	 */
	public void setSelectedNode(String pageId) {
		setSelectedNodePreference(pageId);
		if (isVisible()) {
			IPreferenceNode node = findNodeMatching(getSelectedNodePreference());
			// Select the first node in the tree if the requested node isn't available
			if (node == null) {
				node = getFirstNode();
			}
			if (node != null) {
				selectNodeInTree(node);
				showPage(node);
			}
		}
	}
	
	protected void selectNodeInTree(IPreferenceNode node) {
		TreePath path = findPathOfNode(node);
		if (path != null) {
			getNodeTree().setSelectionPath(path);
		}
	}
	
	protected TreePath findPathOfNode(IPreferenceNode node) {
		TreePath result = null;
		DefaultMutableTreeNode treeNode = PreferenceTreeModel.getTreeNodeForPrefNode(node);
		if (treeNode != null) {
			TreeNode[] path = getPreferenceTreeModel().getPathToRoot(treeNode);
			if (path != null) {
				result = new TreePath(path);
			}
		}
		return result;
	}

	/**
	 * Sets the name of the selected item preference.
	 * 
	 * @param pageId
	 *            The identifier for the page
	 */
	protected void setSelectedNodePreference(String pageId) {
		lastPreferenceId = pageId;
	}
	
	/**
	 * Get the name of the selected item preference
	 * 
	 * @return String
	 */
	protected String getSelectedNodePreference() {
		return lastPreferenceId;
	}
	
	/**
	 * Clear the last selected node. This is so that we not chache the last
	 * selection in case of an error.
	 */
	void clearSelectedNode() {
		setSelectedNodePreference(null);
	}
	
	public void setMessage(String message, IMessageProvider.Type type) {
		Icon newIcon;
		
		switch (type) {
			case ERROR : newIcon = com.onaro.client.leekui.ui.internal.Resources.INSTANCE.getErrorIcon(); break;
			case WARNING : newIcon = com.onaro.client.leekui.ui.internal.Resources.INSTANCE.getWarningIcon(); break;
			case INFO : newIcon = com.onaro.client.leekui.ui.internal.Resources.INSTANCE.getInfoIcon(); break;
			case NONE :
			default: newIcon = null;
		}
		
		getPageDescriptionLabel().setText(message);
		getPageDescriptionLabel().setIcon(newIcon);
	}

	public Preferences getPreferenceStore() {
		return preferenceStore;
	}
	
	public void setPreferenceStore(Preferences preferenceStore) {
		this.preferenceStore = preferenceStore;
	}

	public void updateButtons() {
		getOkButton().setEnabled(currentNode.getPage().isValid());		
	}

	public void updateMessage() {
		IPreferencePage currentPage = currentNode.getPage();
		DialogUtils.consumeMessage(this, currentPage);
	}

	public void updateTitle() {
		IPreferencePage currentPage = currentNode.getPage();
		getPageTitleLabel().setText(currentPage.getTitle());
		getPageTitleLabel().setIcon(currentPage.getIcon());
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return com.jidesoft.dialog.ButtonPanel	
	 */
	private ButtonPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new ButtonPanel();
			buttonPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.gray), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			buttonPanel.add(getOkButton(), ButtonPanel.AFFIRMATIVE_BUTTON);
			buttonPanel.add(getCancelButton(), ButtonPanel.CANCEL_BUTTON);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Messages.INSTANCE.getOKButtonLabel());
			okButton.setName("okButton"); //$NON-NLS-1$
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					okPressed();
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Messages.INSTANCE.getCancelButtonLabel());
			cancelButton.setName("cancelButton"); //$NON-NLS-1$
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					cancelPressed();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setMinimumSize(new Dimension(120, 120));
			jScrollPane.setViewportView(getNodeTree());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes nodeTree	
	 * 	
	 * @return javax.swing.JTree	
	 */
	private JTree getNodeTree() {
		if (nodeTree == null) {
			nodeTree = new JTree();
			nodeTree.setName("nodeTree"); //$NON-NLS-1$
			nodeTree.setModel(getPreferenceTreeModel());
			nodeTree.setCellRenderer(getPreferenceNodeTreeCellRenderer());
			nodeTree.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodeTree.getLastSelectedPathComponent();
					if (node == null || !(node.getUserObject() instanceof IPreferenceNode)) return;
					showPage((IPreferenceNode)node.getUserObject());
				}
			});
			nodeTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			TreeUtils.expandAll(nodeTree);
		}
		return nodeTree;
	}

	/**
	 * This method initializes centerPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getCenterPanel() {
		if (centerPanel == null) {
			centerPanel = new JPanel();
			centerPanel.setLayout(new BorderLayout());
			centerPanel.add(getPageHeaderPanel(), BorderLayout.NORTH);
			centerPanel.add(getPageContentsPanel(), BorderLayout.CENTER);
		}
		return centerPanel;
	}

	/**
	 * This method initializes pageHeaderPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPageHeaderPanel() {
		if (pageHeaderPanel == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 1;
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridy = 2;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 1;
			gridBagConstraints2.insets = new Insets(5, 10, 10, 5);
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints2.gridy = 3;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.insets = new Insets(5, 10, 5, 5);
			gridBagConstraints1.gridy = 1;
			pageHeaderPanel = new JPanel();
			pageHeaderPanel.setLayout(new GridBagLayout());
			pageHeaderPanel.add(getPageTitleLabel(), gridBagConstraints1);
			pageHeaderPanel.add(getPageDescriptionLabel(), gridBagConstraints2);
			pageHeaderPanel.add(getJSeparator(), gridBagConstraints);
		}
		return pageHeaderPanel;
	}

	/**
	 * This method initializes pageTitleLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getPageTitleLabel() {
		if (pageTitleLabel == null) {
			pageTitleLabel = new JLabel();
			// Temporary dummy title
			pageTitleLabel.setText(Messages.INSTANCE.getPageTitleLabel());
			pageTitleLabel.setFont(pageTitleLabel.getFont().deriveFont(Font.BOLD, 14));
		}
		return pageTitleLabel;
	}

	/**
	 * This method initializes pageDescriptionLabel	
	 * 	
	 * @return javax.swing.JLabel	
	 */
	private JLabel getPageDescriptionLabel() {
		if (pageDescriptionLabel == null) {
			pageDescriptionLabel = new JLabel();
			// Temporary dummy description
			pageDescriptionLabel.setText(Messages.INSTANCE.getDescriptionLabel());
		}
		return pageDescriptionLabel;
	}

	/**
	 * This method initializes jSeparator	
	 * 	
	 * @return javax.swing.JSeparator	
	 */
	private JSeparator getJSeparator() {
		if (jSeparator == null) {
			jSeparator = new JSeparator();
		}
		return jSeparator;
	}
	
	static class PreferenceTreeModel extends DefaultTreeModel {
		private static final long serialVersionUID = 1L;
		
		private static final Comparator<IPreferenceNode> PREFERENCE_NODE_COMPARATOR = new Comparator<IPreferenceNode>() {
			public int compare(IPreferenceNode o1, IPreferenceNode o2) {
				return o1.getLabelText().compareToIgnoreCase(o2.getLabelText());
			}
		};
		
		private static Map<IPreferenceNode, DefaultMutableTreeNode> prefNodeToTreeNodeMap = new HashMap<IPreferenceNode, DefaultMutableTreeNode>();

		public PreferenceTreeModel() {
			super(createTree());
		}
		
		private static TreeNode createTree() {
			PreferenceManager preferenceManager = getPreferenceManager();
			prefNodeToTreeNodeMap.clear();
			DefaultMutableTreeNode root = new DefaultMutableTreeNode();
			createChildren(root, preferenceManager.getRootSubNodes());
			return root;
		}
		
		private static void createChildren(DefaultMutableTreeNode parent, List<IPreferenceNode> children) {
			List<IPreferenceNode> sortedChildren = new ArrayList<IPreferenceNode>(children);
			Collections.sort(sortedChildren, PREFERENCE_NODE_COMPARATOR);
			for (IPreferenceNode child : sortedChildren) {
				DefaultMutableTreeNode node = new DefaultMutableTreeNode(child);
				parent.add(node);
				prefNodeToTreeNodeMap.put(child, node);
				createChildren(node, child.getSubNodes());
			}
		}
		
		public static DefaultMutableTreeNode getTreeNodeForPrefNode(IPreferenceNode prefNode) {
			return prefNodeToTreeNodeMap.get(prefNode);
		}
	}
	
	static class PreferenceNodeTreeCellRenderer extends DefaultTreeCellRenderer {
		private static final long serialVersionUID = 1L;

		@Override
		public Icon getClosedIcon() {
			return null;
		}

		@Override
		public Icon getLeafIcon() {
			return null;
		}

		@Override
		public Icon getOpenIcon() {
			return null;
		}

		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
			JLabel renderer = (JLabel)super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			if (node.getUserObject() instanceof IPreferenceNode) {
				IPreferenceNode prefNode = (IPreferenceNode)node.getUserObject();
				
				renderer.setText(prefNode.getLabelText());
			}
			
			return renderer;
		}
	}

	/**
	 * This method initializes preferenceTreeModel	
	 * 	
	 * @return com.onaro.client.leekui.preference.PreferenceDialog.PreferenceTreeModel	
	 */
	private PreferenceDialog.PreferenceTreeModel getPreferenceTreeModel() {
		if (preferenceTreeModel == null) {
			preferenceTreeModel = new PreferenceDialog.PreferenceTreeModel();
		}
		return preferenceTreeModel;
	}

	/**
	 * This method initializes preferenceNodeTreeCellRenderer	
	 * 	
	 * @return com.onaro.client.leekui.preference.PreferenceDialog.PreferenceNodeTreeCellRenderer	
	 */
	private PreferenceDialog.PreferenceNodeTreeCellRenderer getPreferenceNodeTreeCellRenderer() {
		if (preferenceNodeTreeCellRenderer == null) {
			preferenceNodeTreeCellRenderer = new PreferenceDialog.PreferenceNodeTreeCellRenderer();
		}
		return preferenceNodeTreeCellRenderer;
	}
	
	// Add dummy pages for elements at design time
	static {
		if (Beans.isDesignTime()) {		
			getPreferenceManager().addToRoot(new PreferenceNode("page1", Messages.INSTANCE.getPrefPage1(), new BlankPreferencePage(Messages.INSTANCE.getPrefPage1(), "This is Page 1"))); //$NON-NLS-1$ //$NON-NLS-2$
			getPreferenceManager().addToRoot(new PreferenceNode("page2", Messages.INSTANCE.getPrefPage2(), new BlankPreferencePage(Messages.INSTANCE.getPrefPage2(), "This is Page 2"))); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * This method initializes pageContentsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPageContentsPanel() {
		if (pageContentsPanel == null) {
			pageContentsPanel = new JPanel();
			pageContentsPanel.setLayout(getCardLayout());
		}
		return pageContentsPanel;
	}

	/**
	 * This method initializes cardLayout	
	 * 	
	 * @return java.awt.CardLayout	
	 */
	private CardLayout getCardLayout() {
		if (cardLayout == null) {
			cardLayout = new CardLayout();
		}
		return cardLayout;
	}

	/**
	 * This method initializes splitPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getSplitPane() {
		if (splitPane == null) {
			splitPane = new JSplitPane();
			splitPane.setResizeWeight(0.15D);
			splitPane.setLeftComponent(getJScrollPane());
			splitPane.setRightComponent(getCenterPanel());
		}
		return splitPane;
	}
	
	@Override
	public void setVisible(boolean visible) {
		if (preferenceStore != null) {
			// TODO: Use this when DefaultSizePosition gets moved to com.onaro.client.ui plugin
			// DefaultSizePosition.restoreBounds(preferenceStore, this, 0.4, 0.6);
		}
		if (visible) {
			IPreferenceNode node = findNodeMatching(getSelectedNodePreference());
			// Select the first node in the tree if the requested node isn't available
			if (node == null) {
				node = getFirstNode();
			}
			if (node != null) {
				selectNodeInTree(node);
				showPage(node);
			}
		}
		super.setVisible(visible);
	}
	
	private IPreferenceNode getFirstNode() {
		TreeNode rootNode = (TreeNode)getPreferenceTreeModel().getRoot();
		if (rootNode.getChildCount() < 1) return null;
		
		DefaultMutableTreeNode firstChild = (DefaultMutableTreeNode)rootNode.getChildAt(0);
		IPreferenceNode node = (IPreferenceNode)firstChild.getUserObject();
		return node;
	}

	public ExceptionListener getExceptionListener() {
		return exceptionListener;
	}

	public void setExceptionListener(ExceptionListener exceptionListener) {
		this.exceptionListener = exceptionListener;
	}	

}  //  @jve:decl-index=0:visual-constraint="10,10"
