/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.preferences;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * A concrete implementation of a node in a preference dialog tree. This class
 * also supports lazy creation of the node's preference page.
 * 
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.preferences.PreferenceNode
 */
public class PreferenceNode implements IPreferenceNode {
	
	static Logger logger = Logger.getLogger(PreferenceNode.class);
	
    /**
     * Preference page, or <code>null</code> if not yet loaded.
     */
    private IPreferencePage page;

    /**
     * The list of subnodes (immediate children) of this node (element type:
     * <code>IPreferenceNode</code>).
     */
    private List<IPreferenceNode> subNodes;
    
    /**
     * Class that implements <code>IPreferenceNode</code>, or
     * <code>null</code> if none.
     */
    private Class<? extends IPreferencePage> pageClass;

    /**
     * The id of this node.
     */
    private String id;

    /**
     * Text label for this node. Note that this field is only used prior to the
     * creation of the preference page.
     */
    private String label;


    /**
     * Creates a new preference node with the given id. The new node has no
     * subnodes.
     * 
     * @param id
     *            the node id
     */
    public PreferenceNode(String id, String label) {
        if (id == null || id.length() == 0) throw new IllegalArgumentException("ID cannot be null"); //$NON-NLS-1$
        this.id = id;
        this.label = label;
    }
    
    /**
     * Creates a preference node with the given id and preference page. The
     * title of the preference page is used for the node label. The node will
     * not have an image.
     * 
     * @param id
     *            the node id
     * 
     */
    public PreferenceNode(String id, String label, Class<? extends IPreferencePage> pageClass) {
        this(id, label);
        this.pageClass = pageClass;
    }

    /**
     * Creates a preference node with the given id and preference page. The
     * title of the preference page is used for the node label. The node will
     * not have an image.
     * 
     * @param id
     *            the node id
     * @param preferencePage
     *            the preference page
     */
    public PreferenceNode(String id, String label, IPreferencePage preferencePage) {
        this(id, label);
        page = preferencePage;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public void add(IPreferenceNode node) {
        if (subNodes == null) {
			subNodes = new ArrayList<IPreferenceNode>();
		}
        subNodes.add(node);
    }
    
    /**
     * Creates a new instance of the given class <code>className</code>.
     * 
     * @param className
     * @return new Object or <code>null</code> in case of failures.
     */
    protected IPreferencePage createObject() {
		try {
			if (pageClass != null) {
				return pageClass.newInstance();
			}
		} catch (Exception e) {
			String msg = "Failed to create preference page: " + id; //$NON-NLS-1$

			logger.warn(msg + " - " + e.getMessage(), e); //$NON-NLS-1$
		}
		return null;
	}

    /*
	 * (non-Javadoc) Method declared on IPreferenceNode.
	 */
    public void createPage() {
    	if (page == null) {
    		page = createObject();
    	}
    	if (page.getTitle() == null) {
    		page.setTitle(label);
    	}
    }

    /*
     * (non-Javadoc) Method declared on IContributionNode.
     */
    public IPreferenceNode findSubNode(String id) {
    	if (id == null || id.length() == 0) throw new IllegalArgumentException("ID cannot be null"); //$NON-NLS-1$
        if (subNodes == null || subNodes.isEmpty()) {
			return null;
		}
        for (IPreferenceNode node : subNodes) {
            if (id.equals(node.getId())) {
				return node;
			}
        }
        return null;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public String getId() {
        return this.id;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public String getLabelText() {
//        if (page != null) {
//			return page.getTitle();
//		}
        return label;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public IPreferencePage getPage() {
        return page;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public List<IPreferenceNode> getSubNodes() {
        if (subNodes == null) {
			return Collections.emptyList();
		}
        return Collections.unmodifiableList(subNodes);
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public IPreferenceNode remove(String id) {
        IPreferenceNode node = findSubNode(id);
        if (node != null) {
			remove(node);
		}
        return node;
    }

    /*
     * (non-Javadoc) Method declared on IPreferenceNode.
     */
    public boolean remove(IPreferenceNode node) {
        if (subNodes == null) {
			return false;
		}
        return subNodes.remove(node);
    }

    /**
     * Set the current page to be newPage.
     * 
     * @param newPage
     */
    public void setPage(IPreferencePage newPage) {
        page = newPage;
    }
}
