/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import com.jidesoft.dialog.ButtonPanel;
import com.onaro.client.leekui.jface.preferences.IPreferencePage;
import com.onaro.client.leekui.jface.preferences.IPreferencePageContainer;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.preferences.PreferencePage
 */
public abstract class PreferencePage implements IPreferencePage {
	
	protected IPreferencePageContainer preferencePageContainer;
	protected JComponent component;
	protected Icon icon;
	protected String title;
	protected String description;
	protected String message;
	protected String errorMessage;
	protected String warningMessage;
	protected boolean valid = true;
	
	private JPanel content;
	private JButton applyButton;
	
    /**
     * Whether this page has the standard Apply button; 
     * <code>true</code> by default.
     *
     * @see #noApplyButton
     */
    private boolean createApplyButton = true;
    
    /**
     * Suppresses creation of the standard Apply button
     * for this page.
     * <p>
     * Subclasses wishing a preference page wihthout these buttons
     * should call this framework method before the page's control
     * has been created.
     * </p>
     */
    protected void noApplyButton() {
        createApplyButton = false;
    }
    
    /**
     * Returns the Apply button.
     * 
     * @return the Apply button
     */
    protected JButton getApplyButton() {
        return applyButton;
    }
    
    /**
     * Contributes additional buttons to the given panel.
     * <p>
     * The default implementation of this framework hook method does
     * nothing. Subclasses should override this method to contribute buttons 
     * to this page's button bar.
     * </p>
     *
     * @param parent the button bar
     */
    protected void contributeButtons(ButtonPanel parent) {
    }
	
	protected abstract JComponent createComponent();
	
	public final JComponent getComponent(boolean create) {
		if (component == null && create) {
			component = createComponent();
			content = new JPanel(new BorderLayout());
			content.add(component, BorderLayout.CENTER);
			ButtonPanel buttonPanel = new ButtonPanel(SwingConstants.RIGHT, ButtonPanel.SAME_SIZE);
			contributeButtons(buttonPanel);
			if (createApplyButton) {
				applyButton = new JButton(Messages.INSTANCE.getApplyLabel());
				applyButton.setName(ButtonPanel.APPLY);
				applyButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						performApply();
					}
				});
				buttonPanel.add(applyButton, ButtonPanel.OTHER_BUTTON);
			}
			if (buttonPanel.getComponentCount() != 0) {
				content.add(buttonPanel, BorderLayout.SOUTH);
				content.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 5));
			}
		}
		
		return content;
	}

	public Dimension computeSize() {
		if (content != null) {
			return content.getPreferredSize();
		}
		return new Dimension(0,0);
	}

	public boolean isValid() {
		return valid;
	}
	
	public void setValid(boolean valid) {
		if (valid != this.valid) {
			this.valid = valid;
			
			// update container state
			if (getContainer() != null) {
				getContainer().updateButtons();
			}
			// update page state
			updateApplyButton();
		}
	}

	public boolean okToLeave() {
		return isValid();
	}

	public boolean performCancel() {
		// Do nothing by default
		return true;
	}

	public boolean performOk() {
		// Do nothing by default
		return true;
	}
	
	protected void performApply() {
		performOk();
	}
	
    /**
     * Updates the enabled state of the Apply button to reflect whether 
     * this page is valid.
     */
    protected void updateApplyButton() {
        if (applyButton != null) {
			applyButton.setEnabled(isValid());
		}
    }

	public void setContainer(IPreferencePageContainer preferencePageContainer) {
		this.preferencePageContainer = preferencePageContainer;
	}
	
	public IPreferencePageContainer getContainer() {
		return preferencePageContainer;
	}

	public void setSize(Dimension size) {
		if (component != null) {
			component.setPreferredSize(size);
		}
	}

	public void dispose() {
		// Do nothing by default
	}

	public String getDescription() {
		return description;
	}

	public String getErrorMessage() {
		return errorMessage;
	}
	
	public String getWarningMessage() {
		return warningMessage;
	}
	
	public Icon getIcon() {
		return icon;
	}

	public String getMessage() {
		return message;
	}

	public String getTitle() {
		return title;
	}

	public void performHelp() {
		// Do nothing by default
	}

	public void setDescription(String description) {
		if (this.description != description) {
			this.description = description;
			if (getContainer() != null) {
				getContainer().updateMessage();
			}
		}
	}

	public void setIcon(Icon icon) {
		if (this.icon != icon) {
			this.icon = icon;
			if (getContainer() != null) {
				getContainer().updateTitle();
			}
		}
	}

	public void setTitle(String title) {
		if (this.title != title) {
			this.title = title;
			if (getContainer() != null) {
				getContainer().updateTitle();
			}
		}
	}
	
	public void setMessage(String message) {
		if (this.message != message) {
			this.message = message;
			if (getContainer() != null) {
				getContainer().updateMessage();
			}
		}
	}
	
	public void setErrorMessage(String errorMessage) {
		if (this.errorMessage != errorMessage) {
			this.errorMessage = errorMessage;
			if (getContainer() != null) {
				getContainer().updateMessage();
			}
		}
	}
	
	public void setWarningMessage(String warningMessage) {
		if (this.warningMessage != warningMessage) {
			this.warningMessage = warningMessage;
			if (getContainer() != null) {
				getContainer().updateMessage();
			}
		}
	}

	public void setVisible(boolean visible) {
		if (component != null) {
			component.setVisible(visible);
		}
	}
}
