/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.util;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Utility class that allow us to monitor an instance of {@link IProgressMonitor} for possible cancelation
 * of the task being executed. <p>
 * 
 * During contruction an instance of {@link Runnable} is passed with the logic that we wish to run and possible to stop.<br>
 * This runnable instance is used to set an internal Thread which could be "stopped" in case the User pressed the cancel button.
 * 
 * @author john.cohen
 * @version 02/27/08
 */
public class InterruptableMonitorUtil extends Thread {
	private Logger logger = Logger.getLogger(InterruptableMonitorUtil.class);
	private Runnable runnable;
	private IProgressMonitor monitor;
	private String taskName;
	private static final int ELAPSE_TIME = 500;

	/**
	 * Constructor
	 * 
	 * @param taskName identifier of the task
	 * @param runnable task that we wish to run
	 * @param monitor allows us to receive notification when the "cancel" button is pressed.
	 */
	public InterruptableMonitorUtil(final String taskName, final Runnable runnable,  IProgressMonitor monitor) {
		this.taskName = taskName;
		this.runnable = runnable;
		this.monitor = monitor;
	}

	/**
	 * It uses a {@link Runnable} instance passed in at construction time to initialize a {@link Thread} class.<br>
	 * If the user has pressed the "cancel" button we'll stop the Thread instance
	 */
	public void run() {
		// Creates a Thread with the business logic passed in using a Runnable obj.
		final Thread subTaskThread = new Thread(runnable);
		
		try {
			// Runs the logic that the user wishes to execute.
			subTaskThread.start();
			
		} catch (Throwable t) {
			logger.error("[" + taskName + "] - Impossible to run a runnable task - " + t.getMessage(), t);
		} 
	
		

		// Checks the status of the thread that has the logic and the monitor.
		//
		while (true) {
			try {
				// Stops the loop-thread if the sub task thread has finished.
				if (!subTaskThread.isAlive()){
					break;
				}
				
				// Interrupts the sub task thread in case the user has pressed the "cancel" button.
				if (monitor.isCanceled()) {
					subTaskThread.interrupt();
					break;
				}
				
				// Stops the loop-thread if the sub task thread has finished. 
				if (subTaskThread.isInterrupted()) {
					break;
				}

				// Giving the chance for other thread to run.
				Thread.sleep(ELAPSE_TIME);
			} catch (InterruptedException e) {
				break;
			}
		}
		// Call this method in case the user has forgotten to include in the runnable logic.
		monitor.done();
	}
}
