/*******************************************************************************
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
* NetApp, Inc. - Reimplementation as part of the Leekui project
*******************************************************************************/

package com.onaro.client.leekui.jface.wizard;

import com.jidesoft.dialog.ButtonPanel;
import com.onaro.client.leekui.jface.dialogs.IMessageProvider;
import com.onaro.client.leekui.jface.dialogs.TitleAreaDialog;
import com.onaro.client.leekui.jface.preferences.Messages;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

/**
 * This class is a reimplementation of the Eclipse class org.eclipse.jface.wizard.WizardDialog
 */
public class WizardDialog extends TitleAreaDialog implements IWizardContainer {
	
	private static final long serialVersionUID = 1L;

	private IWizard wizard = null;
	
	private IWizardPage currentPage = null;
		
	private ButtonPanel buttonPanel;
	
	private JButton backButton;
	
	private JButton nextButton;
	
	private JButton finishButton;
	
	
	private boolean isMovingToPreviousPage = false;
	
	private JPanel pageContainer;
	
	private CardLayout pageContainerLayout;
	
	public WizardDialog(IWizard wizard) {
		setWizard(wizard);
	}
	
	public WizardDialog(Dialog parent, IWizard wizard) {
		super(parent, null);
		setWizard(wizard);
	}
	
	public WizardDialog(Frame parent, IWizard wizard) {
		super(parent, null);
		setWizard(wizard);
	}
	
	protected void setWizard(IWizard newWizard) {
        this.wizard = newWizard;
        this.wizard.setContainer(this);
        this.setName(this.wizard.getWindowTitle());
        showStartingPage();

        //install ESC key to close the wizard
        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
        Action escapeAction = new AbstractAction(){
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e){
                cancelPressed();
            }
        };
        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE"); //$NON-NLS-1$
        getRootPane().getActionMap().put("ESCAPE", escapeAction); //$NON-NLS-1$
    }
	
	protected IWizard getWizard() {
		return wizard;
	}
	
    /**
     * The Back button has been pressed.
     */
    protected void backPressed() {
        IWizardPage page = currentPage.getPreviousPage();
        if (page == null) {
			// should never happen since we have already visited the page
            return;
		}
        // set flag to indicate that we are moving back
        isMovingToPreviousPage = true;
        // show the page
        showPage(page);
    }
    
    /**
     * The Next button has been pressed.
     */
    protected void nextPressed() {
        IWizardPage page = currentPage.getNextPage();
        if (page == null) {
            // something must have happend getting the next page
            return;
        }
        // show the next page
        showPage(page);
    }
    
    @Override
    protected void cancelPressed() {
    	// Only close the dialog if the wizard approves it.
    	if (wizard.performCancel()) {
    		super.cancelPressed();
    	}
    }
    
    /**
     * The Finish button has been pressed.
     */
    protected void finishPressed() {
        if (wizard.performFinish()) {
            okPressed();
        }
    }

	public IWizardPage getCurrentPage() {
		return currentPage;
	}

	public void showPage(IWizardPage page) {
        if (page == null || page == currentPage) {
            return;
        }
        if (!isMovingToPreviousPage) {
			// remember my previous page.
            page.setPreviousPage(currentPage);
		} else {
			isMovingToPreviousPage = false;
		}

		updateForPage(page);
	}
	
	private void updateForPage(IWizardPage page) {
        // ensure this page belongs to the current wizard
        if (wizard != page.getWizard()) {
			setWizard(page.getWizard());
		}
        // ensure that page control has been created
        // (this allows lazy page control creation)
        if (page.getComponent(false) == null) {
            Component newComponent = page.getComponent(true);
            
            getPageContainer().add(newComponent, page.getName());
            
            // ensure the dialog is large enough for this page
            updateSize(page);
        }
        // make the new page visible
        IWizardPage oldPage = currentPage;
        currentPage = page;
        currentPage.setVisible(true);
        getPageContainerLayout().show(getPageContainer(), currentPage.getName());
        if (oldPage != null) {
        	oldPage.setVisible(false);
        }

        // update the dialog controls
        update();
	}
	
    /**	
     * Shows the starting page of the wizard.
     */
    private void showStartingPage() {
        currentPage = wizard.getStartingPage();
        if (currentPage == null) {
            // something must have happend getting the page
            return;
        }
        updateForPage(currentPage);
    }
	
    /**
     * Updates this dialog's controls to reflect the current page.
     */
    protected void update() {
        // Update the window title
        updateWindowTitle();
        // Update the title bar
        updateTitleBar();
        // Update the buttons
        updateButtons();
		// Fires the page change event
		//firePageChanged(new PageChangedEvent(this, getCurrentPage()));
    }
    
    /* (non-Javadoc)
     * Method declared on IWizardContainer.
     */
    public void updateButtons() {
        boolean canFlipToNextPage = false;
        boolean canFinish = wizard.canFinish();
		getBackButton().setEnabled(currentPage.getPreviousPage() != null);
		
        canFlipToNextPage = currentPage.canFlipToNextPage();
        getNextButton().setEnabled(canFlipToNextPage);
        getFinishButton().setEnabled(canFinish);

        // finish is default unless it is diabled and next is enabled
        if (canFlipToNextPage && !canFinish) {
        	this.getRootPane().setDefaultButton(getNextButton());
		} else {
			this.getRootPane().setDefaultButton(getFinishButton());
		}
    }

	public void updateSize() {
		updateSize(currentPage);		
	}
	
	protected void updateSize(IWizardPage page) {
		if (page == null || page.getComponent(true) == null) {
			return;
		}
		
		Dimension existingSize = getPageContainer().getSize();
		Dimension requestedSize = page.getComponent(true).getPreferredSize();
		
		int newWidth = Math.max(existingSize.width, requestedSize.width);
		int newHeight = Math.max(existingSize.height, requestedSize.height);
		Dimension newSize = new Dimension(newWidth, newHeight);
		
		if (!newSize.equals(existingSize)) {
			getPageContainer().setPreferredSize(new Dimension(newWidth, newHeight));
			pack();
		}
	}
	
	public void updateTitleBar() {
        String s = null;
        if (currentPage != null) {
			s = currentPage.getTitle();
		}
        if (s == null) {
			s = ""; //$NON-NLS-1$
		}
        setTitleText(s);
        if (currentPage != null) {
            setTitleIcon(currentPage.getIcon());
        }
        updateMessage();
	}
    
    /* (non-Javadoc)
     * Method declared on IWizardContainer.
     */
    public void updateMessage() {
        if (currentPage == null) {
			return;
		}

        if (currentPage.getErrorMessage() != null) {
			setMessage(currentPage.getErrorMessage(), IMessageProvider.Type.ERROR);
		} else if (currentPage.getMessage() != null) {
			setMessage(currentPage.getMessage(), IMessageProvider.Type.INFO);
		} else {
			setMessage(currentPage.getDescription(), IMessageProvider.Type.NONE);
		}
    }

	public void updateWindowTitle() {
		String title = wizard.getWindowTitle();
		if (title == null) {
			title = ""; //$NON-NLS-1$
		}
		setTitle(title);
	}
	
	@Override
	protected Component getDialogArea() {
		return getPageContainer();
	}
	
	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return com.jidesoft.dialog.ButtonPanel	
	 */
	protected ButtonPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new ButtonPanel();
			buttonPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.lightGray), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			buttonPanel.setSizeContraint(ButtonPanel.SAME_SIZE);
			buttonPanel.setGroupGap(10);
			buttonPanel.add(getBackButton());
			buttonPanel.add(getNextButton());
			buttonPanel.add(getFinishButton());
			buttonPanel.add(getCancelButton());
		}
		return buttonPanel;
	}
	
	protected JButton getBackButton() {
		if (backButton == null) {
			backButton = new JButton();
            backButton.setMnemonic(KeyEvent.VK_B);
            backButton.setName("backButton"); //$NON-NLS-1$
			backButton.setText(Messages.INSTANCE.getBackButtonLabel());
			backButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					backPressed();
				}
			});
		}
		return backButton;
	}
	
	protected JButton getNextButton() {
		if (nextButton == null) {
			nextButton = new JButton();
            nextButton.setMnemonic(KeyEvent.VK_N);
            nextButton.setName("nextButton"); //$NON-NLS-1$
            nextButton.setText(Messages.INSTANCE.getNextButtonLabel());
			nextButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					nextPressed();
				}
			});
		}
		return nextButton;
	}
	
	/**
	 * This method initializes finishButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	protected JButton getFinishButton() {
		if (finishButton == null) {
			finishButton = new JButton();
            finishButton.setMnemonic(KeyEvent.VK_F);
            finishButton.setName("finishButton"); //$NON-NLS-1$
            finishButton.setText(Messages.INSTANCE.getFinishButtonLabel()); //$NON-NLS-1$
			finishButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					finishPressed();
				}
			});
		}
		return finishButton;
	}

	protected JPanel getPageContainer() {
		if (pageContainer == null) {
			pageContainer = new JPanel();
			pageContainer.setLayout(getPageContainerLayout());
		}
		return pageContainer;
	}
	
	protected CardLayout getPageContainerLayout() {
		if (pageContainerLayout == null) {
			pageContainerLayout = new CardLayout();
		}
		return pageContainerLayout;
	}
	
	public static int show(Window parent, IWizard wizard) {
		return show(parent, wizard, true);
	}
	
	public static int show(Window parent, IWizard wizard, boolean modal) {
		WizardDialog wizardDialog = null;
		if (parent instanceof Frame) {
			wizardDialog = new WizardDialog((Frame)parent, wizard);
		} else if (parent instanceof Dialog) {
			wizardDialog = new WizardDialog((Dialog)parent, wizard);
		} else {
			wizardDialog = new WizardDialog(wizard);
		}
		
		wizardDialog.setModal(modal);
		
		wizardDialog.pack();
		
		Dimension size = wizardDialog.getSize();
		size.height = Math.max(size.height, MINUMUM_SIZE.height);
		size.width = Math.max(size.width, MINUMUM_SIZE.width);
		wizardDialog.setSize(size);
		
		wizardDialog.setLocationRelativeTo(parent);
		wizardDialog.setVisible(true);
		
		return wizardDialog.getReturnCode();
	}
}
